mod cm108;
mod config;

use cm108::*;
use config::*;

use cpal::{
	traits::{DeviceTrait, HostTrait},
	Device,
};
use rodio::{OutputStream, OutputStreamHandle};
use std::thread;
use std::{fs, io::Cursor, time::Instant};

fn main() {
	let args: Vec<_> = std::env::args().collect();
	if args.len() > 1 && args[1].to_lowercase() == "list" {
		let _: Option<()> = devices(|name, _| {
			println!("{}", name);

			None
		});

		std::process::exit(0);
	}

	let conf = get_conf();
	let beep = fs::read(&conf.beep_file).expect("Could not read beep file");
	let callsign = fs::read(&conf.identification_file).expect("Could not read identification file");
	let cm108 = find_cm108_device().expect("Could not find CM108 device");
	let device = devices(|name, dev| {
		if name == conf.audio_device {
			Some(dev)
		} else {
			None
		}
	})
	.unwrap_or_else(|| panic!("Could not find audio device {}", conf.audio_device));
	let (_stream, stream_handle) =
		OutputStream::try_from_device(&device).expect("Could not get stream from audio device");

	let mut last_ident = Instant::now();
	tx_audio(&cm108, &conf, &stream_handle, callsign.clone());
	loop {
		thread::sleep(conf.beacon_period);

		if (Instant::now() - last_ident) > conf.identification_period {
			last_ident = Instant::now();
			tx_audio(&cm108, &conf, &stream_handle, callsign.clone());
		} else {
			tx_audio(&cm108, &conf, &stream_handle, beep.clone());
		}
	}
}

fn devices<F, T>(f: F) -> Option<T>
where
	F: Fn(String, Device) -> Option<T>,
{
	cpal::available_hosts()
		.iter()
		.filter_map(|host_id| {
			cpal::host_from_id(*host_id)
				.ok()
				.map(|host| {
					host.devices()
						.ok()
						.map(|devices| devices.filter_map(|dev| Some((dev.name().ok()?, dev))))
				})
				.flatten()
		})
		.flatten()
		.filter_map(|(name, device)| f(name, device))
		.next()
}

fn tx_audio(
	cm108: &cm108::Thing,
	conf: &Config,
	stream_handle: &OutputStreamHandle,
	beep: Vec<u8>, // have to pass mutably for cursor. Not ideal
) {
	set_ptt(cm108, true);
	thread::sleep(conf.tx_delay);
	stream_handle
		.play_once(Cursor::new(beep))
		.expect("Could not play audio")
		.sleep_until_end();
	set_ptt(cm108, false);
}

fn set_ptt(cm108: &cm108::Thing, state: bool) {
	cm108.set_all_gpio(state).unwrap();
}
